-- Emulate a business select using math.
-- Our criteria are:
-- * Numbers <= 1,000,000
--   * that are multiples of 31
--   * that are multiples of 17
--   * where (Number + 1) is prime
--   * where the digits add to 27
--   * where the product of non-zero digits is a perfect square
--        * Example: 862803.  8*6*2*8*3 = 2304 = 48^2

use CorpDB;
go

drop table if exists #Selection;
go

create table #Selection
(
	CustomerID int not null
);
go

with l0 as (select 1 v union all select 1),
l1 as (select a.v from l0 a, l0),
l2 as (select a.v from l1 a, l1),
l3 as (select a.v from l2 a, l2),
l4 as (select a.v from l3 a, l3),
l5 as (select a.v from l4 a, l4),
nums as (select row_number() over (order by (select null)) num from l5)
insert #Selection (CustomerID)
select n.num as CustomerID
from nums n
where n.num <= 1000000
and n.num % 31 = 0;

with l0 as (select 1 v union all select 1),
l1 as (select a.v from l0 a, l0),
l2 as (select a.v from l1 a, l1),
l3 as (select a.v from l2 a, l2),
l4 as (select a.v from l3 a, l3),
l5 as (select a.v from l4 a, l4),
nums as (select row_number() over (order by (select null)) num from l5)
insert #Selection (CustomerID)
select n.num as CustomerID
from nums n
where n.num <= 1000000
and n.num % 17 = 0;

with l0 as (select 1 v union all select 1),
l1 as (select a.v from l0 a, l0),
l2 as (select a.v from l1 a, l1),
l3 as (select a.v from l2 a, l2),
l4 as (select a.v from l3 a, l3),
l5 as (select a.v from l4 a, l4),
nums as (select row_number() over (order by (select null)) num from l5)
insert #Selection (CustomerID)
select n.num
from nums n
inner join dbo.PrimeNumber p on n.num + 1 = p.Prime
where n.num <= 1000000;

with l0 as (select 1 v union all select 1),
l1 as (select a.v from l0 a, l0),
l2 as (select a.v from l1 a, l1),
l3 as (select a.v from l2 a, l2),
l4 as (select a.v from l3 a, l3),
l5 as (select a.v from l4 a, l4),
nums as (select row_number() over (order by (select null)) num from l5),
SumOfDigits as
(
	select n.num,
		n.num / 1000000 +				-- millions place
		(n.num / 100000) % 10 +			-- hundred thousands place
		(n.num / 10000 % 10) +			-- ten thousands place
		(n.num / 1000 % 10) +			-- thousands place
		(n.num / 100 % 10) +			-- hundreds place
		(n.num / 10 % 10) +				-- tens place
		(n.num % 10) SumOfDigits		-- ones place
	from nums n
	where n.num <= 1000000
)
insert #Selection (CustomerID)
select s.num
from SumOfDigits s
where s.SumOfDigits = 27;

with l0 as (select 1 v union all select 1),
l1 as (select a.v from l0 a, l0),
l2 as (select a.v from l1 a, l1),
l3 as (select a.v from l2 a, l2),
l4 as (select a.v from l3 a, l3),
l5 as (select a.v from l4 a, l4),
nums as (select row_number() over (order by (select null)) num from l5),
ProductOfNonZeroDigits as
(
	select n.num,
		case when n.num / 1000000 = 0 then 1 else n.num / 1000000 end *					
		case when (n.num / 100000) % 10 = 0 then 1 else (n.num / 100000) % 10 end *
		case when (n.num / 10000 % 10) = 0 then 1 else (n.num / 10000 % 10) end *
		case when (n.num / 1000 % 10) = 0 then 1 else (n.num / 1000 % 10) end *
		case when (n.num / 100 % 10) = 0 then 1 else (n.num / 100 % 10) end *
		case when (n.num / 10 % 10) = 0 then 1 else (n.num / 10 % 10) end *
		case when (n.num % 10) = 0 then 1 else (n.num % 10) end ProductOfDigits
	from nums n
	where n.num <= 1000000
), ProductWithSqrt as
(
	select p.num,
		p.ProductOfDigits,
		cast(sqrt(p.ProductOfDigits) as int) SqrtProductOfDigits
	from ProductOfNonZeroDigits p
)
insert #Selection (CustomerID)
select p.num
from ProductWithSqrt p
where power(p.SqrtProductOfDigits, 2) = p.ProductOfDigits;

-- How many rows in the table?
select count(CustomerID) NbrRows from #Selection;
select count(distinct CustomerID) DistinctRows from #Selection;

-- Let's get some stats about duplicates
with Duplicates as
(
	select s.CustomerID, count(*) Nbr
	from #Selection s
	group by s.CustomerID
	having count(*) > 1
)
select dup.Nbr CustomersWithDuplicates, count(*) Nbr
from Duplicates dup
group by dup.Nbr
order by dup.Nbr;
go

-- Make copies of the data

drop table if exists tempdb.dbo.Selection1;
go
select CustomerID into tempdb.dbo.Selection1 from #Selection;
go
drop table if exists tempdb.dbo.Selection2;
go
select identity(int, 1,1) as ID, CustomerID into tempdb.dbo.Selection2 from #Selection;
alter table tempdb.dbo.Selection2 add constraint pk_Selection2 primary key clustered (ID);
go
drop table if exists tempdb.dbo.Selection3;
go
select CustomerID into tempdb.dbo.Selection3 from #Selection;
go
drop table if exists tempdb.dbo.Selection4;
go
select CustomerID into tempdb.dbo.Selection4 from #Selection;
go

-- For reference, how many unique values?
select count(distinct CustomerID) from #Selection;
-- 285977